//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Model/RealModel.h
//! @brief     Defines class RealModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_MODEL_REALDATAMODEL_H
#define BORNAGAIN_GUI_MODEL_MODEL_REALDATAMODEL_H

#include "GUI/Model/Model/SessionModel.h"

class RealItem;

//! The RealModel class is a model to store all imported RealItem's.

class RealModel : public SessionModel {
    Q_OBJECT

public:
    explicit RealModel(QObject* parent = nullptr);

    QVector<SessionItem*> nonXMLItems() const override;
    void readFrom(QXmlStreamReader* reader, MessageService* messageService = nullptr) override;

    RealItem* insertSpecularDataItem();
    RealItem* insertIntensityDataItem();
    RealItem* insertDataItem(int rank);
    QVector<RealItem*> realItems() const;
    QVector<RealItem*> realItems(int rank) const;

    QVector<RealItem*> realItems1D() const;
    QVector<RealItem*> realItems2D() const;

    void remove(RealItem* item);

    QStringList realDataNames() const;

signals:
    void realDataAddedOrRemoved();

private:
    void onRowsChange(const QModelIndex& parent, int, int);
};

#endif // BORNAGAIN_GUI_MODEL_MODEL_REALDATAMODEL_H
