//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Support/XML/UtilXML.h
//! @brief     Defines
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_SUPPORT_XML_UTILXML_H
#define BORNAGAIN_GUI_SUPPORT_XML_UTILXML_H

#include <QVariant>
#include <QXmlStreamWriter> // used in every including file
#include <heinz/Complex.h>
#include <heinz/Vectors3D.h>

class QXmlStreamReader;
class MessageService;
class QUuid;


namespace GUI::Session::XML {

namespace Tags {
constexpr auto Id("id");
constexpr auto TagAttribute("Tag");
constexpr auto ModelTypeAttribute("ModelType");
constexpr auto DisplayNameAttribute("DisplayName");
constexpr auto ParameterTag("Parameter");
constexpr auto BinaryData("BinaryData");
constexpr auto NonSessionItemData("NonSessionItemData");
constexpr auto ParameterNameAttribute("ParName");
constexpr auto ParameterTypeAttribute("ParType");
constexpr auto ParameterValueAttribute("ParValue");
constexpr auto ParameterRoleAttribute("ParRole");
constexpr auto ParameterExtAttribute("ParExt");
constexpr auto ParameterToolAttribute("ParTool");
constexpr auto ExternalPropertyColorAtt("Color");
constexpr auto ExternalPropertyIdentifierAtt("Identifier");
} // namespace Tags

constexpr auto Version("Version");

void gotoEndElementOfTag(QXmlStreamReader* reader, const QString& tag);

//! Write the variant as a complete tag, including the given role
void writeVariant(QXmlStreamWriter* writer, QVariant variant, int role);

//! Write the variant's value as an attribute
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName,
                    const QVariant& variant);

void writeUid(QXmlStreamWriter* writer, const QString& id);
void writeUid(QXmlStreamWriter* writer, const QUuid& id);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, int d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, unsigned d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, double d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeBaseName, const R3& vec);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeBaseName, const complex_t& c);

bool readBoolAttribute(QXmlStreamReader* reader, const QString& attributeName);
unsigned readUIntAttribute(QXmlStreamReader* reader, const QString& attributeName);

void readAttribute(QXmlStreamReader* reader, const QString& attributeName, double* d);
void readAttribute(QXmlStreamReader* reader, const QString& attributeName, QString* s);
void readAttribute(QXmlStreamReader* reader, const QString& attributeName, QColor* c);
void readAttribute(QXmlStreamReader* reader, const QString& attributeBaseName, R3* vec);
void readAttribute(QXmlStreamReader* reader, const QString& attributeBaseName, complex_t* c);

void assertExpectedTag(QXmlStreamReader* reader, const QString& tag);

} // namespace GUI::Session::XML

#endif // BORNAGAIN_GUI_SUPPORT_XML_UTILXML_H
