//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Common/DataItemBundleWidget.h
//! @brief     Defines class DataItemBundleWidget
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2022
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Common/DataItemBundleWidget.h"

DataItemBundleWidget::DataItemBundleWidget(QWidget* parent)
    : QWidget(parent)
{
}
DataItemBundleWidget::~DataItemBundleWidget() = default;

void DataItemBundleWidget::setDataItem(DataItem* dataItem)
{
    setDataItems({dataItem});
}

void DataItemBundleWidget::setDataItems(const QList<DataItem*>& dataItems)
{
    m_dataItems = dataItems;

    // remove nullptrs
    for (DataItem* item : m_dataItems)
        if (!item)
            m_dataItems.removeOne(item);

    // we shouldn't work with empty list
    ASSERT(m_dataItems.size());

    // set nullptr at destruction
    for (DataItem*& item : m_dataItems) {
        disconnect(item, &DataItem::destroyed, this, nullptr);
        connect(item, &DataItem::destroyed, this, [&]() { item = nullptr; });
    }
}

QList<SpecularDataItem*> DataItemBundleWidget::specularDataItems() const
{
    return tItems<SpecularDataItem>();
}

QList<IntensityDataItem*> DataItemBundleWidget::intensityDataItems() const
{
    return tItems<IntensityDataItem>();
}

SpecularDataItem* DataItemBundleWidget::currentSpecularDataItem() const
{
    return tFirstItem<SpecularDataItem>();
}

IntensityDataItem* DataItemBundleWidget::currentIntensityDataItem() const
{
    return tFirstItem<IntensityDataItem>();
}
