//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Mask/MaskViewFactory.cpp
//! @brief     Implements class MaskViewFactory
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Mask/MaskViewFactory.h"
#include "GUI/Model/BaseItem/SessionItem.h"
#include "GUI/Model/Data/ProjectionItems.h"
#include "GUI/Model/Device/MaskItems.h"
#include "GUI/Util/Error.h"
#include "GUI/View/Mask/EllipseView.h"
#include "GUI/View/Mask/LineViews.h"
#include "GUI/View/Mask/MaskAllView.h"
#include "GUI/View/Mask/MaskContainerView.h"
#include "GUI/View/Mask/PolygonPointView.h"
#include "GUI/View/Mask/PolygonView.h"
#include "GUI/View/Mask/RegionOfInterestView.h"

#include <boost/polymorphic_cast.hpp>

using boost::polymorphic_downcast;

IShape2DView* MaskViewFactory::createMaskView(SessionItem* item, ISceneAdaptor* adaptor)
{
    IShape2DView* result(nullptr);

    if (item->hasModelType<MaskContainerItem>())
        result = new MaskContainerView(polymorphic_downcast<MaskContainerItem*>(item));

    else if (item->hasModelType<ProjectionContainerItem>())
        result = new MaskContainerView(polymorphic_downcast<ProjectionContainerItem*>(item));


    else if (item->hasModelType<RectangleItem>())
        result = new RectangleView(polymorphic_downcast<RectangleItem*>(item));


    else if (item->hasModelType<PolygonItem>())
        result = new PolygonView(polymorphic_downcast<PolygonItem*>(item));


    else if (item->hasModelType<PolygonPointItem>())
        result = new PolygonPointView(polymorphic_downcast<PolygonPointItem*>(item));


    else if (item->hasModelType<VerticalLineItem>())
        result = new VerticalLineView(polymorphic_downcast<VerticalLineItem*>(item));


    else if (item->hasModelType<HorizontalLineItem>())
        result = new HorizontalLineView(polymorphic_downcast<HorizontalLineItem*>(item));


    else if (item->hasModelType<EllipseItem>())
        result = new EllipseView(polymorphic_downcast<EllipseItem*>(item));


    else if (item->hasModelType<MaskAllItem>())
        result = new MaskAllView(polymorphic_downcast<MaskAllItem*>(item));


    else if (item->hasModelType<RegionOfInterestItem>())
        result = new RegionOfInterestView(polymorphic_downcast<RegionOfInterestItem*>(item));


    else {
        throw Error("MaskViewFactory::createSampleView() -> Error! "
                    "Can't create a view for "
                    + item->modelType());
    }

    result->setSceneAdaptor(adaptor);

    return result;
}
